<?php

namespace Ardent\UtilBundle\EntityBuilder;

use Exception;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

class Builder
{
    /**
     * @var BaseElement[]
     */
    private array $elements = [];

    public function __construct(
        private readonly SymfonyStyle $io,
    )
    {
    }

    public function add(BaseElement|string $element): self
    {
        if (is_string($element)) {
            $element = new BaseElement($element);
        }
        $this->elements[] = $element;

        return $this;
    }

    private function getPopertyNames(): array
    {
        return array_map(fn(BaseElement $element) => $element->getProperty(), $this->elements);
    }

    private function setProperty(object $object, BaseElement $element): void
    {
        $setter = sprintf('set%s', ucfirst($element->getProperty()));
        if (!method_exists($object, $setter)) {
            throw new Exception(sprintf('Object does not have setter "%s"', $setter));
        }
        $object->$setter($this->io->askQuestion($element->getQuestion()));
    }

    public function setAll(object $object): void
    {
        foreach ($this->elements as $element) {
            $this->setProperty($object, $element);
        }
    }

    public function setByQuestion(object $object): void
    {
        $element = $this->io->askQuestion(new ChoiceQuestion('Select property to modify', $this->elements));
        $getter = sprintf('get%s', ucfirst($element->getProperty()));
        $this->io->writeln(sprintf('Current value: %s', $object->$getter()));
        $this->setProperty($object, $element);
    }

    public function getAllProperties(object $object): array
    {
        $data = [];
        foreach ($this->elements as $element) {
            $getter = sprintf('get%s', ucfirst($element->getProperty()));
            $data[] = [$element->getProperty(), $object->$getter()];
        }
        return $data;
    }
}